@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2008-2011 Mike Massonnet <mmassonnet@xfce.org>
+ *  Copyright (c) 2008-2012 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <config.h>
 #endif
 
+#include <exo/exo.h>
 #include <gtk/gtk.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4util/libxfce4util.h>
@@ -27,6 +28,10 @@
 #include <X11/extensions/XTest.h>
 #include <X11/keysym.h>
 
+#ifdef HAVE_QRENCODE
+#include <qrencode.h>
+#endif
+
 #include "common.h"
 #include "collector.h"
 #include "history.h"
@@ -43,19 +48,25 @@ G_DEFINE_TYPE (ClipmanMenu, clipman_menu, GTK_TYPE_MENU)
 
 struct _ClipmanMenuPrivate
 {
-  GtkWidget            *mi_inhibit;
   GtkWidget            *mi_clear_history;
   ClipmanHistory       *history;
   GSList               *list;
   gboolean              reverse_order;
+#ifdef HAVE_QRENCODE
+  gboolean              show_qr_code;
+#endif
   guint                 paste_on_activate;
+  gboolean              never_confirm_history_clear;
 };
 
 enum
 {
   REVERSE_ORDER = 1,
-  INHIBIT_MENU_ITEM,
+#ifdef HAVE_QRENCODE
+  SHOW_QR_CODE,
+#endif
   PASTE_ON_ACTIVATE,
+  NEVER_CONFIRM_HISTORY_CLEAR,
 };
 
 static void             clipman_menu_finalize           (GObject *object);
@@ -67,6 +78,10 @@ static void             clipman_menu_get_property       (GObject *object,
                                                          guint property_id,
                                                          GValue *value,
                                                          GParamSpec *pspec);
+#ifdef HAVE_QRENCODE
+GdkPixbuf *             clipman_menu_qrcode             (char *text);
+#endif
+
 
 /*
  * Private methods declarations
@@ -78,10 +93,13 @@ static void            _clipman_menu_free_list          (ClipmanMenu *menu);
  * Callbacks declarations
  */
 
+#ifdef HAVE_QRENCODE
+static void		cb_set_qrcode                   (GtkMenuItem *mi,
+                                                         const GdkPixbuf *pixbuf);
+#endif
 static void             cb_set_clipboard                (GtkMenuItem *mi,
                                                          const ClipmanHistoryItem *item);
 static void             cb_clear_history                (ClipmanMenu *menu);
-static void             cb_toggle_inhibit_mi            (ClipmanMenu *menu);
 
 
 
@@ -89,6 +107,28 @@ static void             cb_toggle_inhibit_mi            (ClipmanMenu *menu);
  * Callbacks
  */
 
+#ifdef HAVE_QRENCODE
+static void
+cb_set_qrcode (GtkMenuItem *mi, const GdkPixbuf *pixbuf)
+{
+  GtkClipboard *clipboard;
+  ClipmanCollector *collector;
+  ClipmanHistory *history;
+
+  collector = clipman_collector_get ();
+  clipman_collector_set_is_restoring (collector);
+  g_object_unref (collector);
+
+  history = clipman_history_get ();
+  clipman_history_add_image (history, pixbuf);
+
+  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+  gtk_clipboard_set_image (clipboard, GDK_PIXBUF (pixbuf));
+
+  g_object_unref (history);
+}
+#endif
+
 static void
 cb_set_clipboard (GtkMenuItem *mi, const ClipmanHistoryItem *item)
 {
@@ -184,10 +224,39 @@ cb_set_clipboard (GtkMenuItem *mi, const ClipmanHistoryItem *item)
 static void
 cb_clear_history (ClipmanMenu *menu)
 {
+  gint res;
+  GtkWidget *dialog;
   GtkClipboard *clipboard;
 
-  if (!xfce_dialog_confirm (NULL, GTK_STOCK_YES, NULL, _("Are you sure you want to clear the history?"), NULL))
-    return;
+  if (!menu->priv->never_confirm_history_clear)
+    {
+      dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+                                       GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+                                       _("Are you sure you want to clear the history?"));
+
+
+      {
+#if GTK_CHECK_VERSION(2, 22, 0)
+        GtkWidget *content_area = gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog));
+#else
+        GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+#endif
+        GtkWidget *checkbox = gtk_check_button_new_with_label (_("Don't ask again"));
+        exo_binding_new (G_OBJECT (checkbox), "active", G_OBJECT (menu), "never-confirm-history-clear");
+        gtk_widget_show (checkbox);
+        gtk_container_add (GTK_CONTAINER (content_area), checkbox);
+
+        res = gtk_dialog_run (GTK_DIALOG (dialog));
+        gtk_widget_destroy (dialog);
+
+        if (res != GTK_RESPONSE_YES)
+          {
+            g_object_set (menu, "never-confirm-history-clear", FALSE, NULL);
+            return;
+          }
+      }
+
+    }
 
   clipman_history_clear (menu->priv->history);
 
@@ -198,13 +267,6 @@ cb_clear_history (ClipmanMenu *menu)
   gtk_clipboard_clear (clipboard);
 }
 
-static void
-cb_toggle_inhibit_mi (ClipmanMenu *menu)
-{
-  gboolean toggle_value = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menu->priv->mi_inhibit));
-  g_object_set (menu, "inhibit-menu-item", toggle_value, NULL);
-}
-
 /*
  * Private methods
  */
@@ -213,6 +275,9 @@ static void
 _clipman_menu_update_list (ClipmanMenu *menu)
 {
   GtkWidget *mi, *image;
+#ifdef HAVE_QRENCODE
+  GdkPixbuf *pixbuf;
+#endif
   ClipmanHistoryItem *item;
   const ClipmanHistoryItem *item_to_restore;
   GSList *list, *l;
@@ -264,6 +329,37 @@ _clipman_menu_update_list (ClipmanMenu *menu)
       gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, pos++);
       gtk_widget_show_all (mi);
     }
+
+#ifdef HAVE_QRENCODE
+  /* Draw QR Code if clipboard content is text */
+  if (menu->priv->show_qr_code && item_to_restore && item_to_restore->type == CLIPMAN_HISTORY_TYPE_TEXT)
+    {
+      mi = gtk_separator_menu_item_new ();
+      menu->priv->list = g_slist_prepend (menu->priv->list, mi);
+      gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, pos++);
+      gtk_widget_show_all (mi);
+
+      if ((pixbuf = clipman_menu_qrcode (item_to_restore->content.text)) != NULL)
+        {
+          mi = gtk_image_menu_item_new ();
+          gtk_container_add (GTK_CONTAINER (mi), gtk_image_new_from_pixbuf (pixbuf));
+          g_signal_connect (mi, "activate", G_CALLBACK (cb_set_qrcode), pixbuf);
+          menu->priv->list = g_slist_prepend (menu->priv->list, mi);
+          gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, pos++);
+          gtk_widget_show_all (mi);
+	  g_object_unref(pixbuf);
+        }
+      else
+        {
+          mi = gtk_menu_item_new_with_label (_("Could not generate QR-Code."));
+          menu->priv->list = g_slist_prepend (menu->priv->list, mi);
+          gtk_menu_shell_insert (GTK_MENU_SHELL (menu), mi, pos++);
+          gtk_widget_set_sensitive (mi, FALSE);
+          gtk_widget_show (mi);
+        }
+    }
+#endif
+
   g_slist_free (list);
 
   if (pos == 0)
@@ -325,12 +421,14 @@ clipman_menu_class_init (ClipmanMenuClass *klass)
                                                          FALSE,
                                                          G_PARAM_CONSTRUCT|G_PARAM_READWRITE));
 
-  g_object_class_install_property (object_class, INHIBIT_MENU_ITEM,
-                                   g_param_spec_boolean ("inhibit-menu-item",
-                                                         "InhibitMenuItem",
-                                                         "Toggle the inhibit menu item to TRUE or FALSE",
+#ifdef HAVE_QRENCODE
+  g_object_class_install_property (object_class, SHOW_QR_CODE,
+                                   g_param_spec_boolean ("show-qr-code",
+                                                         "ShowQrCode",
+                                                         "Set to TRUE to display QR-Code in the menu",
                                                          FALSE,
                                                          G_PARAM_CONSTRUCT|G_PARAM_READWRITE));
+#endif
 
   g_object_class_install_property (object_class, PASTE_ON_ACTIVATE,
                                    g_param_spec_uint ("paste-on-activate",
@@ -338,6 +436,13 @@ clipman_menu_class_init (ClipmanMenuClass *klass)
                                                       "Paste the content of a menu item when it is activated",
                                                       0, 2, 0,
                                                       G_PARAM_CONSTRUCT|G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class, NEVER_CONFIRM_HISTORY_CLEAR,
+                                   g_param_spec_boolean ("never-confirm-history-clear",
+                                                         "NeverConfirmHistoryClear",
+                                                         "Set to FALSE to clear the history list with confirmation",
+                                                         FALSE,
+                                                         G_PARAM_CONSTRUCT|G_PARAM_READWRITE));
 }
 
 static void
@@ -357,10 +462,6 @@ clipman_menu_init (ClipmanMenu *menu)
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
-  menu->priv->mi_inhibit = mi = gtk_check_menu_item_new_with_mnemonic (_("_Disable"));
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect_swapped (mi, "toggled", G_CALLBACK (cb_toggle_inhibit_mi), menu);
-
   menu->priv->mi_clear_history = mi = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_clear_history), menu);
@@ -390,15 +491,20 @@ clipman_menu_set_property (GObject *object,
       priv->reverse_order = g_value_get_boolean (value);
       break;
 
-    case INHIBIT_MENU_ITEM:
-      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (priv->mi_inhibit),
-                                      g_value_get_boolean (value));
+#ifdef HAVE_QRENCODE
+    case SHOW_QR_CODE:
+      priv->show_qr_code = g_value_get_boolean (value);
       break;
+#endif
 
     case PASTE_ON_ACTIVATE:
       priv->paste_on_activate = g_value_get_uint (value);
       break;
 
+    case NEVER_CONFIRM_HISTORY_CLEAR:
+      priv->never_confirm_history_clear = g_value_get_boolean (value);
+      break;
+
     default:
       break;
     }
@@ -418,16 +524,55 @@ clipman_menu_get_property (GObject *object,
       g_value_set_boolean (value, priv->reverse_order);
       break;
 
-    case INHIBIT_MENU_ITEM:
-      g_value_set_boolean (value, gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (priv->mi_inhibit)));
-      break;
-
     case PASTE_ON_ACTIVATE:
       g_value_set_uint (value, priv->paste_on_activate);
       break;
 
+    case NEVER_CONFIRM_HISTORY_CLEAR:
+      g_value_set_boolean (value, priv->never_confirm_history_clear);
+      break;
+
     default:
       break;
     }
 }
 
+#ifdef HAVE_QRENCODE
+GdkPixbuf *
+clipman_menu_qrcode (char *text)
+{
+	QRcode *qrcode;
+	GdkPixbuf *pixbuf, *pixbuf_scaled;
+	int i, j, k, rowstride, channels;
+	guchar *pixel;
+	unsigned char *data;
+	
+	qrcode = QRcode_encodeString8bit(text, 0, QR_ECLEVEL_L);
+
+	if (qrcode == NULL)
+		return NULL;
+
+	data = qrcode->data;
+
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, qrcode->width + 2, qrcode->width + 2);
+
+	pixel = gdk_pixbuf_get_pixels (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	channels = gdk_pixbuf_get_n_channels (pixbuf);
+
+	gdk_pixbuf_fill(pixbuf, 0xffffffff);
+	for (i = 1; i <= qrcode->width; i++)
+		for (j = 1; j <= qrcode->width; j++) {
+			for (k = 0; k < channels; k++)
+				pixel[i * rowstride + j * channels + k] = !(*data & 0x1) * 0xff;
+			data++;
+		}
+
+	pixbuf_scaled = gdk_pixbuf_scale_simple (pixbuf, (qrcode->width + 2) * 3, (qrcode->width + 2) * 3, GDK_INTERP_NEAREST);
+
+	QRcode_free(qrcode);
+	g_object_unref(pixbuf);
+	
+	return pixbuf_scaled;
+}
+#endif
