@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2011 Mike Massonnet <mmassonnet@xfce.org>
+ *  Copyright (c) 2009-2012 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -87,6 +87,12 @@ prop_dialog_run (void)
   /* General settings */
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "add-selections")),
                                 DEFAULT_ADD_PRIMARY_CLIPBOARD);
+#ifdef HAVE_QRENCODE
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "show-qr-code")),
+                                DEFAULT_SHOW_QR_CODE);
+#else
+  gtk_widget_hide(GTK_WIDGET (gtk_builder_get_object (builder, "show-qr-code")));
+#endif
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "history-ignore-selections")),
                                 DEFAULT_HISTORY_IGNORE_PRIMARY_CLIPBOARD);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "save-on-quit")),
@@ -98,6 +104,10 @@ prop_dialog_run (void)
 
   xfconf_g_property_bind (xfconf_channel, "/settings/add-primary-clipboard", G_TYPE_BOOLEAN,
                           gtk_builder_get_object (builder, "add-selections"), "active");
+#ifdef HAVE_QRENCODE
+  xfconf_g_property_bind (xfconf_channel, "/settings/show-qr-code", G_TYPE_BOOLEAN,
+                          gtk_builder_get_object (builder, "show-qr-code"), "active");
+#endif
   xfconf_g_property_bind (xfconf_channel, "/settings/history-ignore-primary-clipboard", G_TYPE_BOOLEAN,
                           gtk_builder_get_object (builder, "history-ignore-selections"), "active");
   xfconf_g_property_bind (xfconf_channel, "/settings/save-on-quit", G_TYPE_BOOLEAN,
@@ -155,10 +165,14 @@ prop_dialog_run (void)
                                 DEFAULT_REORDER_ITEMS);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "reverse-order")),
                                 DEFAULT_REVERSE_ORDER);
-  gtk_widget_set_sensitive (GTK_WIDGET (gtk_builder_get_object (builder, "skip-action")),
+  gtk_widget_set_sensitive (GTK_WIDGET (gtk_builder_get_object (builder, "skip-action-1")),
                             DEFAULT_ENABLE_ACTIONS);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "skip-action")),
+  gtk_widget_set_sensitive (GTK_WIDGET (gtk_builder_get_object (builder, "skip-action-2")),
+                            DEFAULT_ENABLE_ACTIONS);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "skip-action-1")),
                                 DEFAULT_SKIP_ACTION_ON_KEY_DOWN);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "skip-action-2")),
+                                !DEFAULT_SKIP_ACTION_ON_KEY_DOWN);
 
   xfconf_g_property_bind (xfconf_channel, "/tweaks/popup-at-pointer", G_TYPE_BOOLEAN,
                           gtk_builder_get_object (builder, "popup-at-pointer"), "active");
@@ -167,9 +181,11 @@ prop_dialog_run (void)
   xfconf_g_property_bind (xfconf_channel, "/tweaks/reverse-menu-order", G_TYPE_BOOLEAN,
                           gtk_builder_get_object (builder, "reverse-order"), "active");
   xfconf_g_property_bind (xfconf_channel, "/settings/enable-actions", G_TYPE_BOOLEAN,
-                          gtk_builder_get_object (builder, "skip-action"), "sensitive");
+                          gtk_builder_get_object (builder, "skip-action-1"), "sensitive");
+  xfconf_g_property_bind (xfconf_channel, "/settings/enable-actions", G_TYPE_BOOLEAN,
+                          gtk_builder_get_object (builder, "skip-action-2"), "sensitive");
   xfconf_g_property_bind (xfconf_channel, "/tweaks/skip-action-on-key-down", G_TYPE_BOOLEAN,
-                          gtk_builder_get_object (builder, "skip-action"), "active");
+                          gtk_builder_get_object (builder, "skip-action-1"), "active");
 
   /* Tweaks tab: paste-on-activate combobox */
   combobox = GTK_WIDGET (gtk_builder_get_object (builder, "combobox-paste-on-activate"));
@@ -269,7 +285,7 @@ cb_show_help (GtkButton *button)
   else
     {
       g_free (docpath);
-      docpath = g_strdup ("http://docs.xfce.org/help.php?package=xfce4-clipman-plugin");
+      docpath = g_strdup ("http://docs.xfce.org/extra/clipman/start");
     }
 
   /* Open documentation in webbrowser */
@@ -514,7 +530,7 @@ cb_reset_actions (GtkButton *button)
                                                GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
                                                GTK_MESSAGE_QUESTION,
                                                GTK_BUTTONS_YES_NO,
-                                               _("<b>Reset actions</b>"), NULL);
+                                               _("<b>Reset actions</b>"));
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                             _("Are you sure you want to reset the actions to the system default values?"));
   res = gtk_dialog_run (GTK_DIALOG (dialog));
